summaryrefslogtreecommitdiff
path: root/BS
diff options
context:
space:
mode:
authorBirte Kristina Friesel <birte.friesel@uos.de>2023-12-01 08:57:58 +0100
committerBirte Kristina Friesel <birte.friesel@uos.de>2023-12-01 08:57:58 +0100
commit87f6ad41589028457803af90f1391cc010ffa583 (patch)
tree780820b64a933e8e78e6a8c9b73f64ae9b0b4b1d /BS
parent27db7a5b221770bedede103410c16462f1f5cff1 (diff)
BS: add overhead measurements
Diffstat (limited to 'BS')
-rw-r--r--BS/Makefile7
-rw-r--r--BS/host/app.c148
-rwxr-xr-xBS/run-paper-strong-rank.sh3
-rwxr-xr-xBS/run-paper-weak.sh6
-rwxr-xr-xBS/run.sh20
-rwxr-xr-xBS/support/timer.h6
6 files changed, 144 insertions, 46 deletions
diff --git a/BS/Makefile b/BS/Makefile
index fae815c..90896a8 100644
--- a/BS/Makefile
+++ b/BS/Makefile
@@ -1,13 +1,18 @@
NR_DPUS ?= 1
NR_TASKLETS ?= 16
PROBLEM_SIZE ?= 8192 # original 2, wtf
+WITH_ALLOC_OVERHEAD ?= 0
+WITH_LOAD_OVERHEAD ?= 0
+WITH_FREE_OVERHEAD ?= 0
+WITH_DPUINFO ?= 0
+SDK_SINGLETHREADED ?= 0
COMMON_INCLUDES := support
HOST_SOURCES := $(wildcard host/*.c)
DPU_SOURCES := $(wildcard dpu/*.c)
COMMON_FLAGS := -Wall -Wextra -g -I${COMMON_INCLUDES}
-HOST_FLAGS := ${COMMON_FLAGS} -std=c11 -O3 `dpu-pkg-config --cflags --libs dpu` -DNR_TASKLETS=${NR_TASKLETS} -DNR_DPUS=${NR_DPUS} -DPROBLEM_SIZE=${PROBLEM_SIZE}
+HOST_FLAGS := ${COMMON_FLAGS} -std=c11 -O3 `dpu-pkg-config --cflags --libs dpu` -DNR_TASKLETS=${NR_TASKLETS} -DNR_DPUS=${NR_DPUS} -DPROBLEM_SIZE=${PROBLEM_SIZE} -DWITH_ALLOC_OVERHEAD=${WITH_ALLOC_OVERHEAD} -DWITH_LOAD_OVERHEAD=${WITH_LOAD_OVERHEAD} -DWITH_FREE_OVERHEAD=${WITH_FREE_OVERHEAD} -DWITH_DPUINFO=${WITH_DPUINFO} -DSDK_SINGLETHREADED=${SDK_SINGLETHREADED}
DPU_FLAGS := ${COMMON_FLAGS} -O2 -DNR_TASKLETS=${NR_TASKLETS}
QUIET = @
diff --git a/BS/run-paper-strong-rank.sh b/BS/run-paper-strong-rank.sh
index 824b279..c2d4f36 100755
--- a/BS/run-paper-strong-rank.sh
+++ b/BS/run-paper-strong-rank.sh
@@ -14,8 +14,7 @@ echo "prim-benchmarks BS strong-rank (dfatool edition)"
echo "Started at $(date)"
echo "Revision $(git describe --always)"
-# >64 are not part of upstream
-for nr_dpus in 128 1 4 16 64; do
+for nr_dpus in 1 4 16 64; do
for nr_tasklets in 1 2 4 8 16; do
echo
if make -B NR_DPUS=${nr_dpus} NR_TASKLETS=${nr_tasklets} BL=10 verbose=1; then
diff --git a/BS/run-paper-weak.sh b/BS/run-paper-weak.sh
index f845e30..a27c547 100755
--- a/BS/run-paper-weak.sh
+++ b/BS/run-paper-weak.sh
@@ -15,11 +15,11 @@ echo "prim-benchmarks BS weak (dfatool edition)"
echo "Started at $(date)"
echo "Revision $(git describe --always)"
-# 256 and 512 are not part of upstream
-for nr_dpus in 256 512 1 4 16 64; do
+for nr_dpus in 1 4 16 64; do
for nr_tasklets in 1 2 4 8 16; do
echo
- if make -B NR_DPUS=${nr_dpus} NR_TASKLETS=${nr_tasklets} BL=10 verbose=1; then
+ # original Makefile sets PROBLEM_SIZE=2, for some reason.
+ if make -B NR_DPUS=${nr_dpus} NR_TASKLETS=${nr_tasklets} BL=10 verbose=1 PROBLEM_SIZE=2; then
i=$(( nr_dpus * 262144 ))
timeout --foreground -k 1m 30m bin/bs_host -w 0 -e 100 -i $i || true
fi
diff --git a/BS/run.sh b/BS/run.sh
index cf32e4f..0c67c93 100755
--- a/BS/run.sh
+++ b/BS/run.sh
@@ -8,15 +8,21 @@ set -e
# -e: number of timed iterations
# -i; ignored, always uses 262144 elements
-echo "prim-benchmarks VA (dfatool edition)"
+(
+
+echo "prim-benchmarks BS (dfatool edition)"
echo "Started at $(date)"
echo "Revision $(git describe --always)"
-for nr_dpus in 1 2 4 8 16 32 64 128 256 512; do
- for nr_tasklets in 1 2 3 4 6 8 10 12 16 20 24; do
- echo
- if make -B NR_DPUS=${nr_dpus} NR_TASKLETS=${nr_tasklets} BL=10; then
- timeout --foreground -k 1m 30m bin/bs_host -w 0 -e 100 || true
- fi
+for i in 262144 16777216; do
+ for nr_dpus in 1 4 8 16 32 64 128 256 512 768 1024 1536 2048 2304 2542; do
+ for nr_tasklets in 8 12 16; do
+ echo
+ if make -B NR_DPUS=${nr_dpus} NR_TASKLETS=${nr_tasklets} BL=10; then
+ timeout --foreground -k 1m 30m bin/bs_host -w 0 -e 100 -i $i || true
+ fi
+ done
done
done
+echo "Completed at $(date)"
+) | tee "log-$(hostname)-ndpus.txt"
diff --git a/BS/support/timer.h b/BS/support/timer.h
index 08027af..ff1ae1b 100755
--- a/BS/support/timer.h
+++ b/BS/support/timer.h
@@ -37,9 +37,9 @@
typedef struct Timer{
- struct timeval startTime[4];
- struct timeval stopTime[4];
- double time[4];
+ struct timeval startTime[7];
+ struct timeval stopTime[7];
+ double time[7];
}Timer;
diff --git a/BS/host/app.c b/BS/host/app.c
index 1160163..bcb22c5 100644
--- a/BS/host/app.c
+++ b/BS/host/app.c
@@ -18,6 +18,17 @@
#include <dpu_probe.h>
#endif
+#if WITH_DPUINFO
+#include <dpu_management.h>
+#include <dpu_target_macros.h>
+#endif
+
+#if SDK_SINGLETHREADED
+#define DPU_ALLOC_PROFILE "nrThreadsPerRank=0"
+#else
+#define DPU_ALLOC_PROFILE NULL
+#endif
+
#define XSTR(x) STR(x)
#define STR(x) #x
@@ -75,18 +86,30 @@ int main(int argc, char **argv) {
struct Params p = input_params(argc, argv);
struct dpu_set_t dpu_set, dpu;
uint32_t nr_of_dpus;
+ uint32_t nr_of_ranks;
uint64_t input_size = INPUT_SIZE;
uint64_t num_querys = p.num_querys;
DTYPE result_host = -1;
DTYPE result_dpu = -1;
- // Create the timer
- Timer timer;
+ // Timer declaration
+ Timer timer;
- // Allocate DPUs and load binary
- DPU_ASSERT(dpu_alloc(NR_DPUS, NULL, &dpu_set));
- DPU_ASSERT(dpu_load(dpu_set, DPU_BINARY, NULL));
- DPU_ASSERT(dpu_get_nr_dpus(dpu_set, &nr_of_dpus));
+ // Allocate DPUs and load binary
+#if !WITH_ALLOC_OVERHEAD
+ DPU_ASSERT(dpu_alloc(NR_DPUS, DPU_ALLOC_PROFILE, &dpu_set));
+ timer.time[0] = 0; // alloc
+#endif
+#if !WITH_LOAD_OVERHEAD
+ DPU_ASSERT(dpu_load(dpu_set, DPU_BINARY, NULL));
+ DPU_ASSERT(dpu_get_nr_dpus(dpu_set, &nr_of_dpus));
+ DPU_ASSERT(dpu_get_nr_ranks(dpu_set, &nr_of_ranks));
+ assert(nr_of_dpus == NR_DPUS);
+ timer.time[1] = 0; // load
+#endif
+#if !WITH_FREE_OVERHEAD
+ timer.time[6] = 0; // free
+#endif
#if ENERGY
struct dpu_probe_t probe;
@@ -94,10 +117,10 @@ int main(int argc, char **argv) {
#endif
// Query number adjustement for proper partitioning
- if(num_querys % (nr_of_dpus * NR_TASKLETS))
- num_querys = num_querys + (nr_of_dpus * NR_TASKLETS - num_querys % (nr_of_dpus * NR_TASKLETS));
+ if(num_querys % (NR_DPUS * NR_TASKLETS))
+ num_querys = num_querys + (NR_DPUS * NR_TASKLETS - num_querys % (NR_DPUS * NR_TASKLETS));
- assert(num_querys % (nr_of_dpus * NR_TASKLETS) == 0 && "Input dimension"); // Allocate input and querys vectors
+ assert(num_querys % (NR_DPUS * NR_TASKLETS) == 0 && "Input dimension"); // Allocate input and querys vectors
DTYPE * input = malloc((input_size) * sizeof(DTYPE));
DTYPE * querys = malloc((num_querys) * sizeof(DTYPE));
@@ -106,20 +129,56 @@ int main(int argc, char **argv) {
create_test_file(input, querys, input_size, num_querys);
// Create kernel arguments
- uint64_t slice_per_dpu = num_querys / nr_of_dpus;
+ uint64_t slice_per_dpu = num_querys / NR_DPUS;
dpu_arguments_t input_arguments = {input_size, slice_per_dpu, 0};
for (unsigned int rep = 0; rep < p.n_warmup + p.n_reps; rep++) {
// Perform input transfers
uint64_t i = 0;
+#if WITH_ALLOC_OVERHEAD
+ if(rep >= p.n_warmup) {
+ start(&timer, 0, 0);
+ }
+ DPU_ASSERT(dpu_alloc(NR_DPUS, DPU_ALLOC_PROFILE, &dpu_set));
+ if(rep >= p.n_warmup) {
+ stop(&timer, 0);
+ }
+#endif
+#if WITH_DPUINFO
+ printf("DPUs:");
+ DPU_FOREACH (dpu_set, dpu) {
+ int rank = dpu_get_rank_id(dpu_get_rank(dpu_from_set(dpu))) & DPU_TARGET_MASK;
+ int slice = dpu_get_slice_id(dpu_from_set(dpu));
+ int member = dpu_get_member_id(dpu_from_set(dpu));
+ printf(" %d(%d.%d)", rank, slice, member);
+ }
+ printf("\n");
+#endif
+#if WITH_LOAD_OVERHEAD
+ if(rep >= p.n_warmup) {
+ start(&timer, 1, 0);
+ }
+ DPU_ASSERT(dpu_load(dpu_set, DPU_BINARY, NULL));
+ if(rep >= p.n_warmup) {
+ stop(&timer, 1);
+ }
+ DPU_ASSERT(dpu_get_nr_dpus(dpu_set, &nr_of_dpus));
+ DPU_ASSERT(dpu_get_nr_ranks(dpu_set, &nr_of_ranks));
+ assert(nr_of_dpus == NR_DPUS);
+#endif
+
// Compute host solution
- start(&timer, 0, 0);
+ if(rep >= p.n_warmup) {
+ start(&timer, 2, 0);
+ }
result_host = binarySearch(input, querys, input_size - 1, num_querys);
- stop(&timer, 0);
+ if(rep >= p.n_warmup) {
+ stop(&timer, 2);
+ }
if (rep >= p.n_warmup) {
- start(&timer, 1, 0);
+ start(&timer, 3, 0);
}
DPU_FOREACH(dpu_set, dpu, i)
@@ -148,13 +207,13 @@ int main(int argc, char **argv) {
DPU_ASSERT(dpu_push_xfer(dpu_set, DPU_XFER_TO_DPU, DPU_MRAM_HEAP_POINTER_NAME, input_size * sizeof(DTYPE), slice_per_dpu * sizeof(DTYPE), DPU_XFER_DEFAULT));
if (rep >= p.n_warmup) {
- stop(&timer, 1);
+ stop(&timer, 3);
}
// Run kernel on DPUs
if (rep >= p.n_warmup)
{
- start(&timer, 2, 0);
+ start(&timer, 4, 0);
#if ENERGY
DPU_ASSERT(dpu_probe_start(&probe));
#endif
@@ -164,7 +223,7 @@ int main(int argc, char **argv) {
if (rep >= p.n_warmup)
{
- stop(&timer, 2);
+ stop(&timer, 4);
#if ENERGY
DPU_ASSERT(dpu_probe_stop(&probe));
#endif
@@ -183,9 +242,9 @@ int main(int argc, char **argv) {
// Retrieve results
if (rep >= p.n_warmup) {
- start(&timer, 3, 0);
+ start(&timer, 5, 0);
}
- dpu_results_t* results_retrieve[nr_of_dpus];
+ dpu_results_t* results_retrieve[NR_DPUS];
i = 0;
DPU_FOREACH(dpu_set, dpu, i)
{
@@ -207,24 +266,53 @@ int main(int argc, char **argv) {
free(results_retrieve[i]);
}
if(rep >= p.n_warmup) {
- stop(&timer, 3);
+ stop(&timer, 5);
+ }
+
+#if WITH_ALLOC_OVERHEAD
+#if WITH_FREE_OVERHEAD
+ if(rep >= p.n_warmup) {
+ start(&timer, 6, 0);
}
+#endif
+ DPU_ASSERT(dpu_free(dpu_set));
+#if WITH_FREE_OVERHEAD
+ if(rep >= p.n_warmup) {
+ stop(&timer, 6);
+ }
+#endif
+#endif
int status = (result_dpu == result_host);
if (status) {
printf("[" ANSI_COLOR_GREEN "OK" ANSI_COLOR_RESET "] results are equal\n");
if (rep >= p.n_warmup) {
- printf("[::] BS NMC | n_dpus=%d n_tasklets=%d e_type=%s n_elements=%lu "
- "| throughput_cpu_MBps=%f throughput_pim_MBps=%f throughput_MBps=%f",
- nr_of_dpus, NR_TASKLETS, XSTR(DTYPE), input_size,
- num_querys * sizeof(DTYPE) / timer.time[0],
- num_querys * sizeof(DTYPE) / timer.time[2],
- num_querys * sizeof(DTYPE) / (timer.time[1] + timer.time[2] + timer.time[3]));
- printf(" throughput_cpu_MOpps=%f throughput_pim_MOpps=%f throughput_MOpps=%f",
- num_querys / timer.time[0],
- num_querys / timer.time[2],
- num_querys / (timer.time[1] + timer.time[2] + timer.time[3]));
- printall(&timer, 3);
+ printf("[::] BS UPMEM | n_dpus=%d n_ranks=%d n_tasklets=%d e_type=%s block_size_B=%d n_elements=%lu b_sdk_singlethreaded=%d ",
+ NR_DPUS, nr_of_ranks, NR_TASKLETS, XSTR(DTYPE), BLOCK_SIZE, input_size, SDK_SINGLETHREADED);
+ printf("| latency_alloc_us=%f latency_load_us=%f latency_cpu_us=%f latency_write_us=%f latency_kernel_us=%f latency_read_us=%f latency_free_us=%f",
+ timer.time[0],
+ timer.time[1],
+ timer.time[2],
+ timer.time[3],
+ timer.time[4],
+ timer.time[5],
+ timer.time[6]);
+ printf(" throughput_cpu_MBps=%f throughput_upmem_kernel_MBps=%f throughput_upmem_total_MBps=%f",
+ input_size * sizeof(DTYPE) / timer.time[2],
+ input_size * sizeof(DTYPE) / (timer.time[4]),
+ input_size * sizeof(DTYPE) / (timer.time[0] + timer.time[1] + timer.time[3] + timer.time[4] + timer.time[5] + timer.time[6]));
+ printf(" throughput_upmem_wxr_MBps=%f throughput_upmem_lwxr_MBps=%f throughput_upmem_alwxr_MBps=%f",
+ input_size * sizeof(DTYPE) / (timer.time[3] + timer.time[4] + timer.time[5]),
+ input_size * sizeof(DTYPE) / (timer.time[1] + timer.time[3] + timer.time[4] + timer.time[5]),
+ input_size * sizeof(DTYPE) / (timer.time[0] + timer.time[1] + timer.time[3] + timer.time[4] + timer.time[5]));
+ printf(" throughput_cpu_MOpps=%f throughput_upmem_kernel_MOpps=%f throughput_upmem_total_MOpps=%f",
+ input_size / timer.time[2],
+ input_size / (timer.time[4]),
+ input_size / (timer.time[0] + timer.time[1] + timer.time[3] + timer.time[4] + timer.time[5] + timer.time[6]));
+ printf(" throughput_upmem_wxr_MOpps=%f throughput_upmem_lwxr_MOpps=%f throughput_upmem_alwxr_MOpps=%f\n",
+ input_size / (timer.time[3] + timer.time[4] + timer.time[5]),
+ input_size / (timer.time[1] + timer.time[3] + timer.time[4] + timer.time[5]),
+ input_size / (timer.time[0] + timer.time[1] + timer.time[3] + timer.time[4] + timer.time[5]));
}
} else {
printf("[" ANSI_COLOR_RED "ERROR" ANSI_COLOR_RESET "] results differ!\n");